@@ -737,7 +737,12 @@ lfJSDM <- function(formula, data, inits, priors,
       class(out.fit) <- "lfJSDM"
 
       # Predict occurrence at new sites. 
-      if (p.occ.re > 0) {X.0 <- cbind(X.0, X.re.0 + 1)}
+      if (p.occ.re > 0) {
+        tmp <- unlist(re.level.names)
+        X.re.0 <- matrix(tmp[c(X.re.0 + 1)], nrow(X.re.0), ncol(X.re.0))
+        colnames(X.re.0) <- x.re.names
+      }
+      if (p.occ.re > 0) {X.0 <- cbind(X.0, X.re.0)}
       out.pred <- predict.lfJSDM(out.fit, X.0, coords.0, ignore.RE = FALSE)
       like.samples <- matrix(NA, N, J.0)
       for (q in 1:N) {
